/*                      LEKCJA05.C
//---------------------------------------------------------------------------
Tematy:
        Struktura budowy danych.
        Definiowanie elementw: 
                         PUNKT, ODCINEK, OKRG, UK, ELIPSA, WYCINEK ELIPSY
//---------------------------------------------------------------------------
Funkcje:
        EXT(212) ulong CreateEntity(   
        EXT(210) void savecount (
//---------------------------------------------------------------------------

   Elementy rysunku s zapisywane w MegaCADzie w formie struktury 
   zdefiniowanej nastpujco:

typedef struct _entity
{
    ushort    type;       // typ elementu
    ulong     id;         // numer identyfikacyjny
    t_attribs attr;       // atrybuty elementu
    t_en_data data;       // dane
} t_entity;


typedef struct _attribs
{
    uchar stl;           // styl linii   (1 -  14)
    uchar pen;           // pisak plotera(0 -  15) uytkownik (1 - 16) !!!
    uchar col;           // kolor        (1 -  15)
    uchar wid;           // grubo linii(1 -   8)
    uchar lay;           // layer        (0 - 255)
    uchar grp;           // grupa        (0 - 255)
} t_attribs;

typedef union _en_data
{
    t_text      txt;     // tekst
    t_point     pnt;     // punkt
    t_line      lin;     // linia
    t_circle    cir;     // okrg
    t_c_arc     arc;     // uk
    t_ellipse   elp;     // elipsa
    t_e_arc     ela;     // wycinek elipsy
    t_polyln    pol;     // polilinia-kontur
    t_polygon   spl;     // zbir punktw
    t_dimension dim;     // wymiarowanie
} t_en_data;
//---------------------------------------------------------------------------
   
   W MegaCAD-zie jest zdefiniowanych 19 podstawowych typw elementw,
   ktrym nadano wasne nazwy.

    E_POINT    : punkt
    E_LINE     : odcinek
    E_CIRCLE   : okrg    
    E_C_ARC    : uk       
    E_ELLIPS   : elipsa 
    E_E_ARC    : wycinek elipsy
    E_POLYLN   : polilinia (kontur)
    E_SPLINE   : linia amana
    E_TXTLIN   : linia tekstu
    E_TXTBLK   : blok tekstu
    E_MACRO    : makro
    E_XHATCH   : kreskowanie
    E_H_DIM    : wymiar poziomy         
    E_V_DIM    : wymiar pionowy 
    E_P_DIM    : wymiar ukony 
    E_A_DIM    : wymiar kta  
    E_D_DIM    : wymiar rednicy  
    E_R_DIM    : wymiar promienia 
    E_T_DIM    : tolerancja wymiaru 

//---------------------------------------------------------------------------

    Podstawowe elementy graficzne uywane w MegaCAD-zie s zdefiniowane 
    jako struktury.  Oto ich definicje (znajduj si one w pliku MEGATYP.H)
    i znaczenie elementw struktur.

PUNKT:
typedef struct _point
{
    double x;            // wsprzdna X
    double y;            // wsprzdna Y    
} t_point;


ODCINEK czyli LINIA:
typedef struct _line
{
    double x1;           // wsprzdna X pocztku
    double y1;           // wsprzdna Y pocztku
    double x2;           // wsprzdna X koca
    double y2;           // wsprzdna Y koca  
} t_line;


OKRG:
typedef struct _circle
{
    double xm;           // wsprzdna X rodka okrgu
    double ym;           // wsprzdna Y rodka okrgu  
    double rd;           // promie
} t_circle;


UK:
typedef struct _c_arc
{
    double xm;           // wsprzdna X rodka okrgu  
    double ym;           // wsprzdna Y rodka okrgu  
    double rd;           // promie          
    double a1;           // kt pocztku uku
    double a2;           // kt koca uku    
} t_c_arc;
    

ELIPSA: 
typedef struct _ellipse
{
    double xm;           // wsprzdna X rodka elipsy
    double ym;           // wsprzdna Y rodka elipsy 
    double ra;           // poowa rednicy 1 osi 
    double rb;           // poowa rednicy 2 osi
    double an;           // kt pooenia osi 1
} t_ellipse;


WYCINEK ELIPSY:
typedef struct _e_arc
{
    double xm;           // wsprzdna X rodka elipsy
    double ym;           // wsprzdna Y rodka elipsy 
    double ra;           // poowa rednicy 1 osi 
    double rb;           // poowa rednicy 2 osi
    double an;           // kt pooenia osi 1
    double a1;           // kt pocztku wycinka w stosunku do osi 1
    double a2;           // kt koca wycinka w stosunku do osi 1   
} t_e_arc;


//---------------------------------------------------------------------------

EXT(212) ulong CreateEntity(   // 
             ushort    type,   // typ elementu
             t_attribs *attr,  // atrybuty (albo NULL)
             void      *data); // dane elementu

Opis:
    Funkcja tworzy zdefiniowany element i przedstawia go na ekranie.
    Jeli parametr 'attr' ma warto NULL element bdzie tworzony z 
    wartociami aktualnie ustawionymi w programie MegaCAD.

Return:
    0L  => w przypadku wystpienia bdu
    ID  => numer identyfikacyjny 'ID' stworzonego elementu

Przykad wywoania:
     CreateEntity(E_POINT,NULL,&punkt);
//---------------------------------------------------------------------------

EXT(210) void savecount (void);            

Opis:
    Funkcja suy do zakoczenia operacji zapisu stworzonych elementw.
    Wszystkie elementy (stworzone lub zmienione) od ostatniego wywoania
    funkcji 'savecount' s traktowane przez 'Undo' i 'Redo' jako "jedna 
    cao".

Przykad wywoania:
    savecount();    


*/
//---------------------------------------------------------------------------
//Andrzej Setman                                            Wiede 18.08.95
//---------------------------------------------------------------------------
#include "std.h"
#include "megatyp.h"
#include "megacad.h"
//---------------------------------------------------------------------------
short main(char *filename, char *args)                
{
   int             f;
   t_point         punkt;
   t_line          linia;          
   t_circle        okrag;
   t_c_arc         luk;
   t_ellipse       elipsa;
   t_e_arc         wyc_elipsy;

// 20 punktw
   for (f=1; f<=20; f++)
   {
      punkt.y= 140+f;
      punkt.x= 8*f;
      CreateEntity(E_POINT,NULL,&punkt);
   }
 
// 20 linii
      linia.x1= 200;
      linia.y1= 150;
      linia.y2= 20;
   for (f=1; f<=20; f++)
   {
      linia.x2= 120+5*f;
      CreateEntity(E_LINE,NULL,&linia);
   }

// 20 okrgw
   for (f=1; f<=20; f++)
   {
      okrag.xm= 20+f;
      okrag.ym= 30+f;
      okrag.rd= 5+f;
      CreateEntity(E_CIRCLE,NULL,&okrag);
   }

// 20 ukw
      luk.xm= 120;
      luk.ym= 50;
   for (f=1; f<=20; f++)
   {
      luk.rd= 5+2*f;
      luk.a1= DEF_PI/24*f;
      luk.a2= DEF_PI/12*f;
      CreateEntity(E_C_ARC,NULL,&luk);
   }

// 20 elips
   for (f=1; f<=20; f++)
   {
      elipsa.xm= 40+f;
      elipsa.ym= 90+f;
      elipsa.ra= f;
      elipsa.rb= 3*f;
      elipsa.an= DEF_PI/36*f;
      CreateEntity(E_ELLIPS,NULL,&elipsa);
   }

// 10 wycinkw elips
   for (f=1; f<=10; f++)
   {
      wyc_elipsy.xm= 140+f;
      wyc_elipsy.ym= 130+f;
      wyc_elipsy.ra= 1+f;
      wyc_elipsy.rb= 5+3*f;
      wyc_elipsy.an= DEF_PI/18*f;
      wyc_elipsy.a1= DEF_PI/18*f;
      wyc_elipsy.a2= DEF_PI/9*f;
      CreateEntity(E_E_ARC,NULL,&wyc_elipsy);
   }
   savecount();    
   return(0);
}
//----------------------------koniec-----------------------------------------
